<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.pedigreeonline.com
 * @since      1.0.0
 *
 * @package    Pedigree_Online
 * @subpackage Pedigree_Online/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pedigree_Online
 * @subpackage Pedigree_Online/includes
 * @author     PedigreeOnline.com, Inc <info@pedigreeonline.com>
 */
class Pedigree_Online {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Pedigree_Online_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	* These are the types of reports we can call
	*
	* @since 1.0.0
	* @access public
	* @var array
	*/
	public static $supported_reports = ['pedigree','offspring','saved_hypomating'];

  /* Constants to use around our project */

	public const CACHE_TIMEOUT = 3600; // 1 hour to cache data

	public const ROOT_DOMAIN = 'pedigreeonline.com'; // Our root domain to determine subdomains from

	public const OPTION_PREFIX = 'pedigreeonline_'; // Prefix our keys and transients with this

	public const VERSION = PEDIGREE_ONLINE_VERSION;


	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PEDIGREE_ONLINE_VERSION' ) ) {
			$this->version = PEDIGREE_ONLINE_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'pedigree-online';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->define_shortcodes();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Pedigree_Online_Loader. Orchestrates the hooks of the plugin.
	 * - Pedigree_Online_i18n. Defines internationalization functionality.
	 * - Pedigree_Online_Admin. Defines all hooks for the admin area.
	 * - Pedigree_Online_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pedigree-online-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pedigree-online-i18n.php';

		/**
		 * The class responsible for defining some common functions within this plugin
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pedigree-online-functions.php';

		/**
		 * The class responsible for defining all shortcodes
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pedigree-online-shortcodes.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pedigree-online-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pedigree-online-public.php';

		$this->loader = new Pedigree_Online_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Pedigree_Online_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Pedigree_Online_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

	  if ( is_admin() ) {

      $plugin_admin = new Pedigree_Online_Admin( $this->get_plugin_name(), $this->get_version() );

      $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
      $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
      $this->loader->add_action( 'admin_init', $plugin_admin, 'define_settings' );
      $this->loader->add_action( 'admin_menu', $plugin_admin, 'define_menu' );
      $this->loader->add_action( 'wp_ajax_pedigreeonline_clear_cache', $plugin_admin, 'ajax_clear_cache' );

    }

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Pedigree_Online_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		// Filter the footer to add custom CSS
		$this->loader->add_action( 'wp_footer', $plugin_public, 'inject_css' );

	}

	/**
	 * Register all of the shortcodes related to the plugin
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_shortcodes() {

		$plugin_shortcodes = new Pedigree_Online_Shortcodes( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_shortcode( 'pedigree_online', $plugin_shortcodes, 'pedigree_shortcode' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Pedigree_Online_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
